--[[--------------------------------------------------------
-- Factionizer, a reputation management tool --
------------------------------------------------------------
CODE INDEX (search on index for fast access):
_01_ Addon Variables
_02_ Addon Startup
_03_ Event Handler
_04_ Addon Initialization
_05_ Slash Handler
_06_ General Helper Functions
_07_ Display faction information
_08_ Faction map
_09_ Faction gain
_10_ Handle update
]]----------------------------------------------------------

-- Addon constants
FIZ_NAME 	= "Factionizer"
FIZ_VNMBR	= 201031			-- Number code for this version

-- Colours
FIZ_HELP_COLOUR		= "|cFFFFFF7F"
FIZ_ERROR_COLOUR	= "|cFFFF0000"
FIZ_UNKNOWN_COLOUR	= "|cFF7F7F7F"
FIZ_HOSTILE_COLOUR	= "|cFF650000"
FIZ_NEUTRAL_COLOUR	= "|cFF9F8302"
FIZ_FRIENDLY_COLOUR	= "|cFF11570B"

--------------------------
-- _01_ Addon Variables --
--------------------------

-- Stored data
FIZ_Data = {}			-- Data saved between sessions
-- Initialization
FIZ_Main = nil			-- Main program window
FIZ_InitComplete = nil
-- Faction information
FIZ_FactionMapping = {}
FIZ_FactionGain = {}

------------------------
-- _02_ Addon Startup --
------------------------

------------------------------------------------------------
function FIZ_OnLoad()
	-- Events monitored by Event Handler
	FIZ_Main = this
	this:RegisterEvent("ADDON_LOADED")
	this:RegisterEvent("PLAYER_ENTERING_WORLD")

	-- Slash commands for CLI
	SLASH_FIZ1 = "/factionizer"
	SLASH_FIZ2 = "/fz"
	SlashCmdList.FIZ = FIZ_SlashHandler
end

------------------------------------------------------------
function FIZ_myAddons()
	-- Register addon with myAddons
	if not (myAddOnsFrame_Register) then return end
	local version = GetAddOnMetadata("Factionizer", "Version");
	local date = GetAddOnMetadata("Factionizer", "X-Date");
	local author = GetAddOnMetadata("Factionizer", "Author");
	local web = GetAddOnMetadata("Factionizer", "X-Website");
	if (version == nil) then
		version = "unknown";
	end
	if (date == nil) then
		date = "unknown";
	end
	if (author == nil) then
		author = "unknown";
	end
	if (web == nil) then
		web = "unknown";
	end

	myAddOnsFrame_Register({
		name = FIZ_NAME,
		version = version,
		releaseDate = date,
		author = author,
		email = "none",
		website = web,
		category = MYADDONS_CATEGORY_UNKNOWN,
		optionsframe = nil,
	})
end


------------------------
-- _03_ Event Handler --
------------------------

function FIZ_OnEvent()
	if (event == "ADDON_LOADED") and (arg1 == FIZ_NAME) then
		FIZ_myAddons()
		FIZ_Init()
		FIZ_Main:UnregisterEvent("ADDON_LOADED")

	elseif (event == "PLAYER_ENTERING_WORLD") then
		if not FIZ_InitComplete then
			FIZ_Init()
		end
		FIZ_Main:UnregisterEvent("PLAYER_ENTERING_WORLD")
		FIZ_Main:RegisterEvent("UPDATE_FACTION")

	elseif (event == "UPDATE_FACTION") then
		FIZ_UpdateFaction(arg1)

	end
end

-------------------------------
function FIZ_OnUpdate()
	if FIZ_InitComplete then return end
	if not FIZ_UpdateRequest then return end
	if (GetTime() < FIZ_UpdateRequest) then return end

	FIZ_Init()
end

-------------------------------
-- _04_ Addon Initialization --
-------------------------------

function FIZ_Init()
	local version = GetAddOnMetadata("Factionizer", "Version");
	if (version == nil) then
		version = "unknown";
	end

	-- create data structures
	if not FIZ_Data then FIZ_Data = {} end

	if not FIZ_Data.ShowMobs then FIZ_Data.ShowMobs = true end
	if not FIZ_Data.ShowQuests then FIZ_Data.ShowQuests = true end
	if not FIZ_Data.ShowInstances then FIZ_Data.ShowInstances = true end

	-- Keep version in configuration file
	FIZ_Data.Version = FIZ_VNMBR

	-- Initialize faction information
	FIZ_InitFactionMap()
	FIZ_InitFactionGains()

	FIZ_InitComplete = true
	FIZ_UpdateRequest = nil
end


------------------------
-- _05_ Slash Handler --
------------------------

function FIZ_SlashHandler(msg)
	if not msg then
		FIZ_ShowFactions()
	else
		local msgLower = string.lower(msg)
		local words = FIZ_GetWords(msg)
		local wordsLower = FIZ_GetWords(msgLower)
		local size = FIZ_TableSize(wordsLower)

		if (size>0) then
			if (wordsLower[0]=="show") then
				local name = FIZ_Concat(words,1)
				FIZ_ShowFactions(name)
			elseif (wordsLower[0]=="help") then
				FIZ_Help()
			elseif (wordsLower[0]=="about") then
				FIZ_About()
			else
				FIZ_ShowFactions(msg)
			end
		else
			FIZ_ShowFactions()
		end
	end
end


-----------------------------------
-- _06_ General Helper Functions --
-----------------------------------

------------------------------------------------------------
function FIZ_Print(msg)
	if not (msg) then return end
	DEFAULT_CHAT_FRAME:AddMessage(msg)
end

------------------------------------------------------------
function FIZ_Debug(msg)
	if not (msg) then return end
	--DEFAULT_CHAT_FRAME:AddMessage(msg)
end

------------------------------------------------------------
function FIZ_TableSize(info)
	local result = 0
	if info then
		for item in pairs(info) do result = result + 1 end
	end
	return result
end

------------------------------------------------------------
function FIZ_GetWords(str)
	local ret = {};
	local pos=0;
	local index=0
	while(true) do
		local word;
		_,pos,word=string.find(str, "^ *([^%s]+) *", pos+1);
		if(not word) then
			return ret;
		end
		ret[index]=word
		index = index+1
	end
end

------------------------------------------------------------
function FIZ_Concat(list, start, stop)
	local ret = "";

	if (start == nil) then start = 0 end
	if (stop == nil) then stop = FIZ_TableSize(list) end

	for i = start,stop do
		if list[i] then
			if (ret ~= "") then ret = ret.." " end
			ret = ret..list[i]
		end
	end
	return ret
end

------------------------------------------------------------
function FIZ_Help()
	FIZ_Print(" ", true)
	FIZ_Print(FIZ_HELP_COLOUR..FIZ_NAME..":|r "..FIZ_TXT.help)
	FIZ_Print(FIZ_HELP_COLOUR..FIZ_TXT.usage..":|r /dc help "..FIZ_TXT.helphelp)
	FIZ_Print(FIZ_HELP_COLOUR..FIZ_TXT.usage..":|r /dc about "..FIZ_TXT.helpabout)
end

------------------------------------------------------------
function FIZ_About()
	local ver = GetAddOnMetadata("Factionizer", "Version");
	local date = GetAddOnMetadata("Factionizer", "X-Date");
	local author = GetAddOnMetadata("Factionizer", "Author");

	if (author ~= nil) then
		FIZ_Print(FIZ_NAME.." "..FIZ_TXT.by.." "..FIZ_HELP_COLOUR..author.."|r", true);
	end
	if (ver ~= nil) then
		FIZ_Print("  "..FIZ_TXT.version..": "..FIZ_HELP_COLOUR..ver.."|r", true);
	end
	if (date ~= nil) then
		FIZ_Print("  "..FIZ_TXT.date..": "..FIZ_HELP_COLOUR..date.."|r", true);
	end
end

-----------------------------------
-- _07_ Display faction information  --
-----------------------------------
function FIZ_ShowFactions(faction)
	local ret = false
	local origFaction = ""

	if (faction) then
		origFaction = faction
		faction = string.lower(faction)
		if (FIZ_FactionMapping[faction]) then
			faction = FIZ_FactionMapping[faction]
		end
		FIZ_Print(FIZ_HELP_COLOUR..FIZ_NAME..":|r ".."Looking up faction ["..FIZ_HELP_COLOUR..origFaction.."|r]")
	else
		FIZ_Print(FIZ_HELP_COLOUR..FIZ_NAME..":|r ".."Listing all factions")
	end
	for factionIndex = 1, GetNumFactions() do
		name, description, standingId, bottomValue, topValue, earnedValue, atWarWith,
		canToggleAtWar, isHeader, isCollapsed, isWatched = GetFactionInfo(factionIndex)

		local mappedName = string.lower(name)
		if (FIZ_FactionMapping[mappedName]) then
			mappedName = FIZ_FactionMapping[mappedName]
		end
		if ((faction == nil) or (faction==mappedName)) then
			ret = true
			if isHeader == nil then
				local colour = FIZ_UNKNOWN_COLOUR
				if ((standingId>0) and (standingId<4)) then
					colour = FIZ_HOSTILE_COLOUR
				elseif ((standingId>4) and (standingId<9)) then
					colour = FIZ_FRIENDLY_COLOUR
				else
					colour = FIZ_NEUTRAL_COLOUR
				end

				local standing = getglobal("FACTION_STANDING_LABEL"..standingId)

				local normMax = topValue - bottomValue
				local normCurrent = earnedValue - bottomValue

				local repToNext = topValue - earnedValue
				local repToMax = 43000 - earnedValue

				FIZ_Print(colour.."Faction: "..factionIndex..": is "..name.." with standing "..standingId..": "..standing.." ("..bottomValue.."/"..earnedValue.."/"..topValue..") -> "..normCurrent.."/"..normMax.." -> needed "..repToNext)

				-- look up rep info
				if (FIZ_FactionGain[mappedName]) then
					if (FIZ_FactionGain[mappedName][standingId]) then
						if (FIZ_FactionGain[mappedName][standingId].mobs and FIZ_Data.ShowMobs) then
							for i = 0, FIZ_FactionGain[mappedName][standingId].mobs.count do
								local toDo = ceil(repToNext / FIZ_FactionGain[mappedName][standingId].mobs.data[i].rep)
								if (FIZ_FactionGain[mappedName][standingId].mobs.data[i].zone) then
									FIZ_Print(colour.."   [Mob] "..FIZ_FactionGain[mappedName][standingId].mobs.data[i].name.." ("..FIZ_FactionGain[mappedName][standingId].mobs.data[i].zone..") , "..FIZ_FactionGain[mappedName][standingId].mobs.data[i].rep..": "..toDo.."x")
								else
									FIZ_Print(colour.."   [Mob] "..FIZ_FactionGain[mappedName][standingId].mobs.data[i].name..", "..FIZ_FactionGain[mappedName][standingId].mobs.data[i].rep..": "..toDo.."x")
								end
							end
						end

						if (FIZ_FactionGain[mappedName][standingId].quests and FIZ_Data.ShowQuests) then
							for i = 0, FIZ_FactionGain[mappedName][standingId].quests.count do
								local toDo = ceil(repToNext / FIZ_FactionGain[mappedName][standingId].quests.data[i].rep)
								if (FIZ_FactionGain[mappedName][standingId].quests.data[i].item and
								    FIZ_FactionGain[mappedName][standingId].quests.data[i].num_item) then
								    	local num = FIZ_FactionGain[mappedName][standingId].quests.data[i].num_item * toDo
									FIZ_Print(colour.."   [Quest] "..FIZ_FactionGain[mappedName][standingId].quests.data[i].name..", "..FIZ_FactionGain[mappedName][standingId].quests.data[i].rep..", "..FIZ_FactionGain[mappedName][standingId].quests.data[i].num_item.."x "..FIZ_FactionGain[mappedName][standingId].quests.data[i].item.." (Total: "..num.."): "..toDo.."x"
									)
								else
									FIZ_Print(colour.."   [Quest] "..FIZ_FactionGain[mappedName][standingId].quests.data[i].name..", "..FIZ_FactionGain[mappedName][standingId].quests.data[i].rep..": "..toDo.."x"
									)
								end
							end
						end

						if (FIZ_FactionGain[mappedName][standingId].instance and FIZ_Data.ShowInstances) then
							for i = 0, FIZ_FactionGain[mappedName][standingId].instance.count do
								local toDo = string.format("%.2f", repToNext / FIZ_FactionGain[mappedName][standingId].instance.data[i].rep)
								FIZ_Print(colour.."   [Instance] "..FIZ_FactionGain[mappedName][standingId].instance.data[i].name.." ("..FIZ_FactionGain[mappedName][standingId].instance.data[i].level.."), "..FIZ_FactionGain[mappedName][standingId].instance.data[i].rep..": "..toDo.."x"
								)
							end
						end
					end
				end
			else
				if (isCollapsed) then
					FIZ_Print("Faction "..factionIndex..": "..name.." is a header. It is collapsed.");
				else
					FIZ_Print("Faction "..factionIndex..": "..name.." is a header. It is expanded.");
				end
			end
		end
	end

	if (faction and not ret) then
		FIZ_Print(FIZ_HELP_COLOUR..FIZ_NAME..":|r ".."Faction ["..FIZ_ERROR_COLOUR..origFaction.."|r] is not known to this player")
	end
	return ret
end

-----------------------------------
-- _08_ Faction map              --
-----------------------------------
function FIZ_InitFactionMap()
	if ((GetLocale() == "enGB") or
	    (GetLocale() == "enUS")) then
	    	-- Alliance
	    	FIZ_AddMapping("Darnassus", "Darnassus")
	    	FIZ_AddMapping("Gnomeregan Exiles", "Gnomeregan Exiles")
	    	FIZ_AddMapping("Ironforge", "Ironforge")
	    	FIZ_AddMapping("Stormwind", "Stormwind")
	    	FIZ_AddMapping("Exodar", "Exodar")
	    	-- Horde
	    	FIZ_AddMapping("Darkspear Trolls", "Darkspear Trolls")
	    	FIZ_AddMapping("Orgrimmar", "Orgrimmar")
	    	FIZ_AddMapping("Thunder Bluff", "Thunder Bluff")
	    	FIZ_AddMapping("Undercity", "Undercity")
	    	FIZ_AddMapping("Silvermoon", "Silvermoon")
	    	-- Alliance PvP
	    	FIZ_AddMapping("League of Arathor", "League of Arathor")
	    	FIZ_AddMapping("Silverwing Sentinels", "Silverwing Sentinels")
	    	FIZ_AddMapping("Stormpike Guard", "Stormpike Guard")
	    	-- Horde PvP
	    	FIZ_AddMapping("Defilers", "Defilers")
	    	FIZ_AddMapping("Frostwolf Clan", "Frostwolf Clan")
	    	FIZ_AddMapping("Warsong Outriders", "Warsong Outriders")
	    	-- Steamwheedle Cartel
	    	FIZ_AddMapping("Booty Bay", "Booty Bay")
	    	FIZ_AddMapping("Everlook", "Everlook")
	    	FIZ_AddMapping("Gadgetzan", "Gadgetzan")
	    	FIZ_AddMapping("Ratchet", "Ratchet")
	    	-- Outland
	    	FIZ_AddMapping("Cenarion Expedition", "Cenarion Expedition")
	    	FIZ_AddMapping("Honor Hold", "Honor Hold")
	    	FIZ_AddMapping("Kurenai", "Kurenai")
	    	FIZ_AddMapping("Mag'har", "Mag'har")
	    	FIZ_AddMapping("Ogri'la", "Ogri'la")
	    	FIZ_AddMapping("Netherwing", "Netherwing")
	    	FIZ_AddMapping("Sporeggar", "Sporeggar")
	    	FIZ_AddMapping("The Consortium", "The Consortium")
	    	FIZ_AddMapping("Thrallmar", "Thrallmar")
	    	-- Shattrath
	    	FIZ_AddMapping("Lower City", "Lower City")
	    	FIZ_AddMapping("Sha'tari Skyguard", "Sha'tari Skyguard")
	    	FIZ_AddMapping("The Aldor", "The Aldor")
	    	FIZ_AddMapping("The Scryers", "The Scryers")
	    	FIZ_AddMapping("The Sha'tar", "The Sha'tar")
	    	-- Raid
	    	FIZ_AddMapping("Ashtongue Deathsworn", "Ashtongue Deathsworn")
	    	FIZ_AddMapping("Brood of Nozdormu", "Brood of Nozdormu")
	    	FIZ_AddMapping("Cenarion Circle", "Cenarion Circle")
	    	FIZ_AddMapping("Hydraxian Waterlords", "Hydraxian Waterlords")
	    	FIZ_AddMapping("Scale of the Sands", "Scale of the Sands")
	    	FIZ_AddMapping("Zandalar Tribe", "Zandalar Tribe")
	    	-- Others
	    	FIZ_AddMapping("Argent Dawn", "Argent Dawn")
	    	FIZ_AddMapping("Bloodsail Buccaneers", "Bloodsail Buccaneers")
	    	FIZ_AddMapping("Darkmoon Faire", "Darkmoon Faire")
	    	FIZ_AddMapping("Gelkis Clan Centaur", "Gelkis Clan Centaur")
	    	FIZ_AddMapping("Keepers of Time", "Keepers of Time")
	    	FIZ_AddMapping("Magram Clan Centaur", "Magram Clan Centaur")
	    	FIZ_AddMapping("Ravenholdt", "Ravenholdt")
	    	FIZ_AddMapping("Shen'dralar", "Shen'dralar")
	    	FIZ_AddMapping("Syndicate", "Syndicate")
	    	FIZ_AddMapping("Thorium Brotherhood", "Thorium Brotherhood")
	    	FIZ_AddMapping("Timbermaw Hold", "Timbermaw Hold")
	    	FIZ_AddMapping("The Violet Eye", "The Violet Eye")
	    	FIZ_AddMapping("Wintersaber Trainers", "Wintersaber Trainers")

	elseif (GetLocale() == "deDE") then
		-- German localisation of factions still needs to be finished

	    	-- Alliance
	    	--FIZ_AddMapping("Darnassus", "Darnassus")
	    	--FIZ_AddMapping("Gnomeregan Exiles", "Gnomeregan Exiles")
	    	FIZ_AddMapping("Ironforge", "Eisenschmiede")
	    	FIZ_AddMapping("Stormwind", "Sturmwind")
	    	--FIZ_AddMapping("Exodar", "Exodar")
	    	-- Horde
	    	--FIZ_AddMapping("Darkspear Trolls", "Darkspear Trolls")
	    	--FIZ_AddMapping("Orgrimmar", "Orgrimmar")
	    	--FIZ_AddMapping("Thunder Bluff", "Thunder Bluff")
	    	--FIZ_AddMapping("Undercity", "Undercity")
	    	--FIZ_AddMapping("Silvermoon", "Silvermoon")
	    	-- Alliance PvP
	    	--FIZ_AddMapping("League of Arathor", "League of Arathor")
	    	--FIZ_AddMapping("Silverwing Sentinels", "Silverwing Sentinels")
	    	--FIZ_AddMapping("Stormpike Guard", "Stormpike Guard")
	    	-- Horde PvP
	    	--FIZ_AddMapping("Defilers", "Defilers")
	    	--FIZ_AddMapping("Frostwolf Clan", "Frostwolf Clan")
	    	--FIZ_AddMapping("Warsong Outriders", "Warsong Outriders")
	    	-- Steamwheedle Cartel
	    	--FIZ_AddMapping("Booty Bay", "Booty Bay")
	    	--FIZ_AddMapping("Everlook", "Everlook")
	    	--FIZ_AddMapping("Gadgetzan", "Gadgetzan")
	    	--FIZ_AddMapping("Ratchet", "Ratchet")
	    	-- Outland
	    	--FIZ_AddMapping("Cenarion Expedition", "Cenarion Expedition")
	    	--FIZ_AddMapping("Honor Hold", "Honor Hold")
	    	--FIZ_AddMapping("Kurenai", "Kurenai")
	    	--FIZ_AddMapping("Mag'har", "Mag'har")
	    	--FIZ_AddMapping("Ogri'la", "Ogri'la")
	    	--FIZ_AddMapping("Netherwing", "Netherwing")
	    	--FIZ_AddMapping("Sporeggar", "Sporeggar")
	    	--FIZ_AddMapping("The Consortium", "The Consortium")
	    	--FIZ_AddMapping("Thrallmar", "Thrallmar")
	    	-- Shattrath
	    	--FIZ_AddMapping("Lower City", "Lower City")
	    	--FIZ_AddMapping("Sha'tari Skyguard", "Sha'tari Skyguard")
	    	--FIZ_AddMapping("The Aldor", "The Aldor")
	    	--FIZ_AddMapping("The Scryers", "The Scryers")
	    	--FIZ_AddMapping("The Sha'tar", "The Sha'tar")
	    	-- Raid
	    	--FIZ_AddMapping("Ashtongue Deathsworn", "Ashtongue Deathsworn")
	    	--FIZ_AddMapping("Brood of Nozdormu", "Brood of Nozdormu")
	    	--FIZ_AddMapping("Cenarion Circle", "Cenarion Circle")
	    	--FIZ_AddMapping("Hydraxian Waterlords", "Hydraxian Waterlords")
	    	--FIZ_AddMapping("Scale of the Sands", "Scale of the Sands")
	    	--FIZ_AddMapping("Zandalar Tribe", "Zandalar Tribe")
	    	-- Others
	    	--FIZ_AddMapping("Argent Dawn", "Argent Dawn")
	    	--FIZ_AddMapping("Bloodsail Buccaneers", "Bloodsail Buccaneers")
	    	--FIZ_AddMapping("Darkmoon Faire", "Darkmoon Faire")
	    	--FIZ_AddMapping("Gelkis Clan Centaur", "Gelkis Clan Centaur")
	    	--FIZ_AddMapping("Keepers of Time", "Keepers of Time")
	    	--FIZ_AddMapping("Magram Clan Centaur", "Magram Clan Centaur")
	    	--FIZ_AddMapping("Ravenholdt", "Ravenholdt")
	    	--FIZ_AddMapping("Shen'dralar", "Shen'dralar")
	    	--FIZ_AddMapping("Syndicate", "Syndicate")
	    	--FIZ_AddMapping("Thorium Brotherhood", "Thorium Brotherhood")
	    	--FIZ_AddMapping("Timbermaw Hold", "Timbermaw Hold")
	    	--FIZ_AddMapping("The Violet Eye", "The Violet Eye")
	    	--FIZ_AddMapping("Wintersaber Trainers", "Wintersaber Trainers")

	elseif (GetLocale() == "frFR") then
		-- French localisation of factions still needs to be done
	elseif (GetLocale() == "koKR") then
		-- Korean localisation of factions still needs to be done
	elseif (GetLocale() == "zhCN") then
		-- Chinese (trad) localisation of factions still needs to be done
	elseif (GetLocale() == "zhTW") then
		-- Chinese (simpl) localisation of factions still needs to be done
	elseif (GetLocale() == "ruRU") then
		-- Russian localisation of factions still needs to be done
	elseif (GetLocale() == "esES") then
		-- Spanish localisation of factions still needs to be done
	end
end

function FIZ_AddMapping(a, b)
	if (not FIZ_FactionMapping) then
		FIZ_FactionMapping = {}
	end
	FIZ_FactionMapping[string.lower(a)] = string.lower(b)
end

-----------------------------------
-- _09_ Faction gains            --
-----------------------------------
function FIZ_InitFactionGains()
	if (GetLocale() == "deDE") then
	elseif (GetLocale() == "frFR") then
	elseif (GetLocale() == "esES") then
	elseif (GetLocale() == "ruRU") then
	elseif (GetLocale() == "koKR") then
	elseif (GetLocale() == "zhCN") then
	elseif (GetLocale() == "zhW") then
	else	-- default for enGB, enUS and any case not listed above

		-- --- Alliance ---
		-- Darnassus
		FIZ_AddQuest("Darnassus", 4, 8, "[60] Additional Runecloth", 75, "Runecloth", 20)
		-- Exodar
		FIZ_AddQuest("Exodar", 4, 8, "[60] Additional Runecloth", 75, "Runecloth", 20)
		-- Gnomeregan Exiles
		FIZ_AddQuest("Gnomeregan Exiles", 4, 8, "[60] Additional Runecloth", 75, "Runecloth", 20)
		-- Stormwind
		FIZ_AddQuest("Stormwind", 4, 8, "[60] Additional Runecloth", 75, "Runecloth", 20)
		-- Ironforge (dummy entries)
		FIZ_AddQuest("Ironforge", 4, 8, "[60] Additional Runecloth", 75, "Runecloth", 20)

		-- --- Horde ---
		-- Darkspear Trolls
		FIZ_AddQuest("Darkspear Trolls", 4, 8, "[60] Additional Runecloth", 75, "Runecloth", 20)
		-- Orgrimmar
		FIZ_AddQuest("Orgrimmar", 4, 8, "[60] Additional Runecloth", 75, "Runecloth", 20)
		-- Silvermoon City
		FIZ_AddQuest("Silvermoon City", 4, 8, "[60] Additional Runecloth", 75, "Runecloth", 20)
		-- Thunder Bluff
		FIZ_AddQuest("Thunder Bluff", 4, 8, "[60] Additional Runecloth", 75, "Runecloth", 20)
		-- Undercity
		FIZ_AddQuest("Undercity", 4, 8, "[60] Additional Runecloth", 75, "Runecloth", 20)

		-- --- Alliance PvP ---
		-- Silverwing Sentinels
		-- Stormpike Guard
		-- The League of Arathor

		-- --- Horde PvP ---
		-- Frostwolf Clan
		-- The Defilers
		-- Warsong Outriders

		-- --- Others ---
	    	-- Argent Dawn
	    	FIZ_AddQuest("Argent Dawn", 4, 8, "[55] Minion's Scourgestone", 25, "Minion's Scourgestone", 20)
	    	FIZ_AddQuest("Argent Dawn", 4, 8, "[55] Invader's Scourgestones", 25, "Invader's Scourgestones ", 10)
	    	FIZ_AddQuest("Argent Dawn", 4, 8, "[55] Corruptor's Scourgestone", 25, "Corruptor's Scourgestone", 1)
	    	FIZ_AddQuest("Argent Dawn", 4, 8, "Using Valor Token", 50, "Argent Dawn Valor Token", 1)
	    	FIZ_AddQuest("Argent Dawn", 4, 8, "Insigna of the Dawn", 10)
	    	FIZ_AddQuest("Argent Dawn", 4, 8, "Insigna of the Crusade", 10)
	    	FIZ_AddQuest("Argent Dawn", 4, 8, "Cauldron Quests", 25)
		FIZ_AddMob("Argent Dawn", 4, 5, "Non-Elites in Plaguelands", 5)
		FIZ_AddMob("Argent Dawn", 4, 6, "Elites & Shadowmages in Plaguelands", 5)
		FIZ_AddMob("Argent Dawn", 4, 6, "Cauldron Lords", 12)
		FIZ_AddMob("Argent Dawn", 4, 7, "Stratholme & Scholomance Trash", 5)
		FIZ_AddMob("Argent Dawn", 4, 8, "Stratholme & Scholomance Bosses", 25)
	    	-- Bloodsail Buccaneers
	    	FIZ_AddMob("Bloodsail Buccaneers", 4, 8, "Booty Bay Bruiser & Elite", 25, "Booty Bay")
	    	FIZ_AddMob("Bloodsail Buccaneers", 4, 8, "Some Booty Bay mobs", 5, "Booty Bay")
	    	FIZ_AddMob("Bloodsail Buccaneers", 4, 8, "Most Booty Bay mobs", 1, "Booty Bay")
	    	FIZ_AddMob("Bloodsail Buccaneers", 4, 8, "Most Faldir's Cove mobs", 5, "Arathi Highlands, Faldir's Cove")
	    	FIZ_AddMob("Bloodsail Buccaneers", 4, 8, "Jazzrik & Rigglefuzz", 1, "Badlands")
	    	-- Darkmoon Faire
	    	FIZ_AddQuest("Darkmoon Faire", 4, 4, "Small Furry Paw", 250, "", 5)		-- 500/3000
	    	FIZ_AddQuest("Darkmoon Faire", 4, 4, "Torn Bear Pelt", 250, "", 5)		-- 1100
	    	FIZ_AddQuest("Darkmoon Faire", 4, 4, "Soft Bushy Tail", 250, "", 5)		-- 1700
	    	FIZ_AddQuest("Darkmoon Faire", 4, 4, "Vibrant Plumes", 250, "", 5)		-- 2500
	    	FIZ_AddQuest("Darkmoon Faire", 4, 4, "Evil Bat Eyes", 250, "", 10)		-- always
	    	FIZ_AddQuest("Darkmoon Faire", 4, 4, "Glowing Scorpid Blood", 250, "", 10)	-- always
	    	FIZ_AddQuest("Darkmoon Faire", 4, 4, "Coarse Weightstone", 250, "", 10)		-- 500
	    	FIZ_AddQuest("Darkmoon Faire", 4, 4, "Heavy Grinding Stone", 250, "", 7)	-- 1100
	    	FIZ_AddQuest("Darkmoon Faire", 4, 4, "Green Iron Bracers", 250, "", 3)		-- 1700
	    	FIZ_AddQuest("Darkmoon Faire", 4, 4, "Big Black Mace", 250, "", 1)		-- 2500
	    	FIZ_AddQuest("Darkmoon Faire", 4, 4, "Dense Grinding Stone", 250, "", 8)	-- always
	    	FIZ_AddQuest("Darkmoon Faire", 4, 4, "Copper Modulator", 250, "", 5)		-- 500
	    	FIZ_AddQuest("Darkmoon Faire", 4, 4, "Whirring Bronze Gizmo", 250, "", 7)	-- 1100
	    	FIZ_AddQuest("Darkmoon Faire", 4, 4, "Green Firework", 250, "", 36)		-- 1700
	    	FIZ_AddQuest("Darkmoon Faire", 4, 4, "Mechanical Repair Kit", 250, "", 6)	-- 2500
	    	FIZ_AddQuest("Darkmoon Faire", 4, 4, "Throium Widget", 250, "", 6)		-- always
	    	FIZ_AddQuest("Darkmoon Faire", 4, 4, "Embossed Leather Boots", 250, "", 3)	-- 500
	    	FIZ_AddQuest("Darkmoon Faire", 4, 4, "Toughened Leather Armor", 250, "", 3)	-- 1100
	    	FIZ_AddQuest("Darkmoon Faire", 4, 4, "Barbaric Harness", 250, "", 3)		-- 1700
	    	FIZ_AddQuest("Darkmoon Faire", 4, 4, "Turtle Scale Leggins", 250, "", 1)	-- 2500
	    	FIZ_AddQuest("Darkmoon Faire", 4, 4, "Rugged Armor Kit", 250, "", 8)		-- always
	    	-- Gelkis Clan Centaur
	    	FIZ_AddMob("Gelkis Clan Centaur", 4, 8, "Magram Clan Centaurs", 20)
	    	-- Keepers of Time
	    	FIZ_AddInstance("Keepers of Time", 4, 8, "Old Hillsbarad", 900, false)
	    	FIZ_AddInstance("Keepers of Time", 4, 8, "Black Morass", 1110, false)
	    	-- Magram Clan Centaur
	    	FIZ_AddMob("Magram Clan Centaur", 4, 8, "Gelkis Clan Centaurs", 20)
	    	-- Ravenholdt
	    	FIZ_AddQuest("Ravenholdt", 4, 4, "[24] Syndicate Emblems", 250, "Syndicate Emblem", 1)
	    	FIZ_AddQuest("Ravenholdt", 4, 8, "[50] Junkboxes Needed", 75, "Heavy Junkbox", 5)
	    	-- Shen'dralar
	    	FIZ_AddQuest("Shen'dralar", 4, 8, "Librams in Dire Maul", 500)
	    	FIZ_AddQuest("Shen'dralar", 4, 8, "Class Books in Dire Maul", 500)
	    	-- Syndicate
	    	FIZ_AddMob("Syndicate", 4, 8, "Myrokos Silentform", 5)
	    	FIZ_AddMob("Syndicate", 4, 8, "Winstone Wolfe", 1)
	    	FIZ_AddMob("Syndicate", 4, 8, "Lord Jorach Ravenholdt", 1)
	    	FIZ_AddMob("Syndicate", 4, 8, "Fahrad", 1)
	    	FIZ_AddMob("Syndicate", 4, 8, "Zan Shivsproket", 1)
	    	FIZ_AddMob("Syndicate", 4, 8, "Smudge Thunderwood", 1)
	    	FIZ_AddMob("Syndicate", 4, 8, "Simone Cantrell", 1)
	    	FIZ_AddMob("Syndicate", 4, 8, "Master Kang", 1)
	    	FIZ_AddMob("Syndicate", 4, 8, "Carlo Aurelius", 1)
	    	FIZ_AddMob("Syndicate", 4, 8, "Ravenholdt Assassin", 1)
	    	FIZ_AddMob("Syndicate", 4, 8, "Ravenholdt Guard", 1)
	    	-- Thorium Brotherhood
	    	FIZ_AddQuest("Thorium Brotherhood", 4, 4, "[45] Restoring Fiery Flux", 25, "2x Incendosaur Scales, 1x Coal, Kingsblood", 2)
	    	FIZ_AddQuest("Thorium Brotherhood", 4, 4, "[45] Restoring Fiery Flux", 25, "2x Incendosaur Scales, 1x Coal, Iron Bar", 2)
	    	FIZ_AddQuest("Thorium Brotherhood", 4, 4, "[45] Restoring Fiery Flux", 25, "2x Incendosaur Scales, 1x Coal, Heavy Leather", 2)
	    	FIZ_AddQuest("Thorium Brotherhood", 5, 5, "[45] Gaining Acceptance", 25, "Dark Iron Residue", 4)
	    	FIZ_AddQuest("Thorium Brotherhood", 6, 8, "[60] Favour Amoungst the Brotherhood", 75, "Dark Iron Ore", 10)
	    	FIZ_AddQuest("Thorium Brotherhood", 6, 8, "[60] Favour Amoungst the Brotherhood", 350, "Core Leather", 2)
	    	FIZ_AddQuest("Thorium Brotherhood", 6, 8, "[60] Favour Amoungst the Brotherhood", 500, "Blood of the Mountain", 1)
	    	FIZ_AddQuest("Thorium Brotherhood", 6, 8, "[60] Favour Amoungst the Brotherhood", 500, "Fiery Core", 1)
	    	FIZ_AddQuest("Thorium Brotherhood", 6, 8, "[60] Favour Amoungst the Brotherhood", 500, "Lava Core", 1)
	    	-- Timbermaw Hold (rep divided by a factor of 5 if grey -> how to include this?)
	    	FIZ_AddQuest("Timbermaw Hold", 2, 8, "[55] Feathers for Nafien", 75, "Deadwood Headdress Feathers", 5)
	    	FIZ_AddQuest("Timbermaw Hold", 2, 8, "[55] Feathers for Grazle", 75, "Deadwood Headdress Feathers", 5)
	    	FIZ_AddQuest("Timbermaw Hold", 2, 8, "[56] Beads for Salfa", 75, "Winterfall Spirit Beads", 5)
	    	FIZ_AddMob("Timbermaw Hold", 2, 6, "Deadwood Avenger, Den Watcher, Shaman, Gardener, Pathfinder & Warrior", 5)
	    	FIZ_AddMob("Timbermaw Hold", 2, 6, "Winterfell Runner, Den Watcher, Pathfinder, Totemic, Shaman, & Ursa", 5)
	    	FIZ_AddMob("Timbermaw Hold", 2, 8, "Chieftain Bloodmaw", 15)
	    	FIZ_AddMob("Timbermaw Hold", 2, 8, "Overlord Ror", 15)
	    	FIZ_AddMob("Timbermaw Hold", 2, 8, "Ragepaw (Rare)", 25)
	    	FIZ_AddMob("Timbermaw Hold", 2, 8, "Grizzle Snowpaw (Rare)", 25)
	    	FIZ_AddMob("Timbermaw Hold", 2, 8, "High Chief Winterfall", 25)
	    	-- Wintersaber Trainers
	    	FIZ_AddQuest("Wintersaber Trainers", 4, 8, "[60] Frostsaber Provisions", 75)
	    	FIZ_AddQuest("Wintersaber Trainers", 4, 8, "[60] Winterfall Intrusions", 75)
	    	FIZ_AddQuest("Wintersaber Trainers", 6, 8, "[60] Rampaging Giants", 75)

	    	-- --- Raid ---
	    	-- Ashtongue Deathsworn
	    	   --> Black Temple
	    	-- Brood of Nozdormu
	    	FIZ_AddQuest("Brood of Nozdormu", 1, 4, "[60] The Hand of the Righteous", 200)
	    	FIZ_AddMob("Brood of Nozdormu", 1, 4, "Anubisath Defender", 100, "Temple of Ahn'Qiraj")
	    	FIZ_AddMob("Brood of Nozdormu", 1, 4, "Anubisath Sentinel", 100, "Temple of Ahn'Qiraj")
	    	FIZ_AddMob("Brood of Nozdormu", 1, 4, "Obsidian Eradicator", 100, "Temple of Ahn'Qiraj")
	    	FIZ_AddMob("Brood of Nozdormu", 1, 4, "Anubisath Qiraj Lasher", 100, "Temple of Ahn'Qiraj")
	    	FIZ_AddMob("Brood of Nozdormu", 1, 4, "Vekniss Hive Crawler", 100, "Temple of Ahn'Qiraj")
	    	FIZ_AddMob("Brood of Nozdormu", 1, 4, "Vekniss Soldier", 100, "Temple of Ahn'Qiraj")
	    	FIZ_AddMob("Brood of Nozdormu", 1, 4, "Vekniss Stinger", 100, "Temple of Ahn'Qiraj")
	    	FIZ_AddMob("Brood of Nozdormu", 1, 4, "Vekniss Warrior", 100, "Temple of Ahn'Qiraj")
	    	FIZ_AddMob("Brood of Nozdormu", 1, 4, "Vekniss Wasp", 100, "Temple of Ahn'Qiraj")
	    	FIZ_AddMob("Brood of Nozdormu", 1, 8, "Most Bosses", 50, "Ruins of Ahn'Qiraj")
	    	FIZ_AddMob("Brood of Nozdormu", 1, 8, "Ossirian", 100, "Ruins of Ahn'Qiraj")
	    	FIZ_AddQuest("Brood of Nozdormu", 1, 8, "Qiraji Lord's Insignia", 500)		-- item
	    	FIZ_AddQuest("Brood of Nozdormu", 1, 8, "Ancient Qiraji Artifact", 1000)	-- item
	    	-- Cenarion Circle
	    	FIZ_AddQuest("Cenarion Circle", 4, 8, "[60] Field Duty", 75)
	    	FIZ_AddQuest("Cenarion Circle", 4, 8, "[60] Abyssal Crests", 75, "Abyssal Crest", 3)
	    	FIZ_AddQuest("Cenarion Circle", 4, 8, "[60] Abyssal Signets", 250, "Abyssal Signet", 3)
	    	FIZ_AddQuest("Cenarion Circle", 4, 8, "[60] Abyssal Scepters", 150, "Abyssal Scepter", 1)
	    	FIZ_AddQuest("Cenarion Circle", 4, 8, "[60] Secret Communication", 250, "Enctypted Twilight Text", 10)
	    	FIZ_AddQuest("Cenarion Circle", 4, 8, "[60] Allegiance to Cenarion Circle", 500)
	    	FIZ_AddInstance("Cenarion Circle", 4, 8, "Ruins of Ahn'Qiraj", 1200) -- counting only bosses
	    	FIZ_AddInstance("Cenarion Circle", 4, 8, "Temple of Ahn'Qiraj", nil) -- no data for full run or bosses
	    	FIZ_AddMob("Cenarion Circle", 4, 5, "Twilight Cultist", 1)	-- only 0.2 @ 70
	    	FIZ_AddMob("Cenarion Circle", 4, 6, "Twilight Flamereaver", 1)	-- only 0.2 @ 70
	    	FIZ_AddMob("Cenarion Circle", 4, 6, "Twilight Keeper", 1)	-- only 0.2 @ 70
	    	FIZ_AddMob("Cenarion Circle", 4, 8, "Twilight Lord Everun", 5)	-- only 1 @ 70
	    	FIZ_AddMob("Cenarion Circle", 4, 8, "Abyssal Templar", 5)
	    	FIZ_AddMob("Cenarion Circle", 4, 8, "Abyssal Duke", 25)
	    	FIZ_AddMob("Cenarion Circle", 4, 8, "Abyssal Lord", 50)
	    	-- Hydraxian Waterlords
	    	FIZ_AddMob("Hydraxian Waterlords", 4, 5, "Desert Rumbler, Dust Stormer", 5, "Silithus")
	    	FIZ_AddMob("Hydraxian Waterlords", 4, 5, "Greater Obsidian Elemental", 5, "Burning Steppes")
	    	FIZ_AddMob("Hydraxian Waterlords", 4, 6, "Lord Incendius", 15, "BRD")
	    	FIZ_AddMob("Hydraxian Waterlords", 4, 6, "Huricanian (Rare)", 25, "Silithus")
	    	FIZ_AddMob("Hydraxian Waterlords", 4, 6, "Pyroguard Emberseer", 50, "UBRS")
	    	FIZ_AddMob("Hydraxian Waterlords", 4, 6, "Molten Core Trash", 20, "MC")
	    	FIZ_AddMob("Hydraxian Waterlords", 4, 6, "Molten Destroyer & Lava Pack", 40, "MC")
	    	FIZ_AddInstance("Hydraxian Waterlords", 4, 7, "Molten Core (all bosses)",1050, "MC")
	    	FIZ_AddInstance("Hydraxian Waterlords", 8, 8, "Molten Core (Golemagg & Ragnaros)",350, "MC")
	    	-- Scale of the Sands
	    	FIZ_AddQuest("Scale of the Sands", 4, 4, "[70] The Vial's of Eternity", 3000)
	    	FIZ_AddQuest("Scale of the Sands", 5, 8, "Battle for Mount Hyal", 2600)
	    	-- Zandalar Tribe
	    	FIZ_AddQuest("Zandalar Tribe", 4, 8, "[60] Zulian, Razzashi, Hakkari Coins", 25)
	    	FIZ_AddQuest("Zandalar Tribe", 4, 8, "[60] Sandfury, Skullsplitter, Bloodscalp Coins", 25)
	    	FIZ_AddQuest("Zandalar Tribe", 4, 8, "[60] Gurubashi, Vilebranch, Witherbark Coins", 25)
	    	FIZ_AddQuest("Zandalar Tribe", 6, 8, "[60] The Savage Guard", 500)
	    	FIZ_AddQuest("Zandalar Tribe", 4, 8, "Destroy a Bijou", 75)
	    	FIZ_AddQuest("Zandalar Tribe", 4, 8, "Zandalar Honor Token", 50)
	    	FIZ_AddInstance("Zandalar Tribe", 4, 8, "Zul'Gurub", 2750)
	    	-- The Violet Eye
	    	   --> Karazhan

		-- --- Outland ---
	    	-- Cenarion Expedition
	    	FIZ_AddInstance("Cenarion Expedition", 4, 5, "Slave Pens", nil, false)	-- Trash 5, Bosses 70, no info on full clear
	    	FIZ_AddInstance("Cenarion Expedition", 7, 8, "Slave Pens", nil, true)	-- Trash 5, Bosses 70, no info on full clear
	    	FIZ_AddInstance("Cenarion Expedition", 4, 5, "Underbog", nil, false)	-- Trash 7, Bosses 70, no info on full clear
	    	FIZ_AddInstance("Cenarion Expedition", 7, 8, "Underbog", nil, true)	-- Trash 7, Bosses 70, no info on full clear
	    	FIZ_AddInstance("Cenarion Expedition", 4, 8, "Steamvault", 1662, false)
	    	FIZ_AddQuest("Cenarion Expedition", 4, 5, "[63] Plants of Zangarmarsh", 250, "Unidentified Plant Parts", 10)
	    	FIZ_AddQuest("Cenarion Expedition", 4, 8, "[63] Uncatalogued Species", 500, "Uncatalogued Species", 1)
	    	FIZ_AddQuest("Cenarion Expedition", 4, 8, "[67] Preparing for War", 75, "Coilfang Armaments", 1)
	    	FIZ_AddMob("Cenarion Expedition", 4, 4, "Steam Pump Overseer", 1, "Zangarmarsh")
	    	FIZ_AddMob("Cenarion Expedition", 4, 4, "Wrekt Slave", 2.5, "Zangarmarsh")
	    	FIZ_AddMob("Cenarion Expedition", 4, 4, "Dredghood Drudge", 2.5, "Zangarmarsh")
	    	FIZ_AddMob("Cenarion Expedition", 4, 4, "Bloodscale Overseer", 5, "Zangarmarsh")
	    	FIZ_AddMob("Cenarion Expedition", 4, 4, "Bloodscale Wavecaller", 5, "Zangarmarsh")
	    	FIZ_AddMob("Cenarion Expedition", 4, 4, "Darkcrest Sorceress", 5, "Zangarmarsh")
	    	FIZ_AddMob("Cenarion Expedition", 4, 4, "Darkcrest Slaver", 5, "Zangarmarsh")
	    	FIZ_AddMob("Cenarion Expedition", 4, 4, "Terrorclaw", 7, "Zangarmarsh")
	    	-- Honor Hold
	    	FIZ_AddInstance("Honor Hold", 4, 5, "Hellfire Ramparts", 600, false)
	    	FIZ_AddInstance("Honor Hold", 7, 8, "Hellfire Ramparts", 2000, true)
	    	FIZ_AddInstance("Honor Hold", 4, 5, "Blood Furnace", 750, false)
	    	FIZ_AddInstance("Honor Hold", 7, 8, "Blood Furnace", 2700, true)
	    	FIZ_AddInstance("Honor Hold", 4, 8, "Shattered Halls", 1600, false)
	    	FIZ_AddQuest("Honor Hold", 4, 7, "[60] Hellfire Fortifications", 150)
	    	FIZ_AddMob("Honor Hold", 4, 4, "Shattered Hands", 5, "Hellfire Peninsula")
	    	-- Kurenai
	    	FIZ_AddQuest("Kurenai", 4, 8, "[67] Fierce Enemies", 500, "Obsidian Warbeads", 10)
	    	FIZ_AddMob("Kurenai", 4, 8, "Boulderfist Crusher & Mystic", 10, "Nagrand, Burning Blade Ruins")
	    	FIZ_AddMob("Kurenai", 4, 8, "Kil'sorrow Deathsworn, Cultist & Spellbinder", 10, "Nagrand, Kil'sorrow Fortress")
	    	FIZ_AddMob("Kurenai", 4, 8, "Boulderfist Mage & Warrior", 10, "Nagrand, North & Southwind Clefts")
	    	FIZ_AddMob("Kurenai", 4, 8, "Murkblood Scavenger", 2, "Nagrand, Sunspring Post")
	    	FIZ_AddMob("Kurenai", 4, 8, "Murkblood Brute, Putrifier & Raider", 10, "Nagrand, Sunspring Post")
	    	FIZ_AddMob("Kurenai", 4, 8, "Warmaul Reaver & Shaman", 10, "Nagrand, Laughing Skull Ruins")
	    	FIZ_AddMob("Kurenai", 4, 8, "Warmaul Brute & Warlock", 10, "Nagrand, Warmaul Hill")
	    	-- Mag'har
	    	FIZ_AddQuest("Mag'har", 4, 8, "[67] Proving Your Strength", 500, "Obsidian Warbeads", 10)
	    	FIZ_AddMob("Mag'har", 4, 8, "Boulderfist Crusher & Mystic", 10, "Nagrand, Burning Blade Ruins")
	    	FIZ_AddMob("Mag'har", 4, 8, "Kil'sorrow Deathsworn, Cultist & Spellbinder", 10, "Nagrand, Kil'sorrow Fortress")
	    	FIZ_AddMob("Mag'har", 4, 8, "Boulderfist Mage & Warrior", 10, "Nagrand, North & Southwind Clefts")
	    	FIZ_AddMob("Mag'har", 4, 8, "Murkblood Scavenger", 2, "Nagrand, Sunspring Post")
	    	FIZ_AddMob("Mag'har", 4, 8, "Murkblood Brute, Putrifier & Raider", 10, "Nagrand, Sunspring Post")
	    	FIZ_AddMob("Mag'har", 4, 8, "Warmaul Reaver & Shaman", 10, "Nagrand, Laughing Skull Ruins")
	    	FIZ_AddMob("Mag'har", 4, 8, "Warmaul Brute & Warlock", 10, "Nagrand, Warmaul Hill")
	    	-- Ogri'la
	    	FIZ_AddQuest("Ogri'la", 4, 8, "[70] The Relic's Emanation", 250)
	    	FIZ_AddQuest("Ogri'la", 4, 8, "[70] Wrangle Some Aether Rays!", 350)
	    	FIZ_AddQuest("Ogri'la", 4, 8, "[70] Bombing Run", 500)
	    	FIZ_AddQuest("Ogri'la", 6, 8, "[70] Banish the Demons", 350)
	    	-- Netherwing
	    	-- todo
	    	-- Sporeggar
	    	FIZ_AddQuest("Sporeggar", 3, 4, "[63] The Sporeling's Plight", 750, "Mature Spore Sac", 10)
	    	FIZ_AddQuest("Sporeggar", 3, 4, "[64] Natural Enemies", 750, "Bog Lord Tendril", 6)
	    	FIZ_AddQuest("Sporeggar", 4, 4, "[64] Glowcap Mushrooms", 750, "Glowcap", 10)
	    	FIZ_AddQuest("Sporeggar", 5, 8, "[64] Now that we're Friends...", 750)
	    	FIZ_AddQuest("Sporeggar", 4, 8, "[64] Fertile Spores", 750, "Fertile Spores", 10)
	    	FIZ_AddQuest("Sporeggar", 5, 8, "[65] Bring me a Shrubbery!", 750, "Sanguine Hibiscus", 5)
		-- Consortium
		FIZ_AddInstance("The Consortium", 4, 5, "Mana Tombs", 1200, false)
		FIZ_AddInstance("The Consortium", 6, 8, "Mana Tombs", 1200, true)
		FIZ_AddQuest("The Consortium", 4, 4, "[66] Stealing from Thieves", 250, "Oshu'gun Crystal Fragment", 10)
		FIZ_AddQuest("The Consortium", 4, 4, "[66] A Head Full of Ivory", 250, "Pair of Ivory Tusks", 3)
		FIZ_AddQuest("The Consortium", 5, 8, "[67] Obsidian Warbeads", 250, "Obsidian Warbeads", 10)
		FIZ_AddQuest("The Consortium", 5, 8, "[68] A Heap of Ethereals", 250, "Zaxxis Insigna", 10)
		FIZ_AddQuest("The Consortium", 6, 8, "[70] Etherum Secrets", 250, "Etherum Priosoner I.D. Tag", 1)
		FIZ_AddQuest("The Consortium", 7, 8, "[70] A Thousand Worlds", 500, "Etherum Prison Key", 5)
	    	-- Thrallmar
	    	FIZ_AddInstance("Thrallmar", 4, 5, "Hellfire Ramparts", 600, false)
	    	FIZ_AddInstance("Thrallmar", 7, 8, "Hellfire Ramparts", 2000, true)
	    	FIZ_AddInstance("Thrallmar", 4, 5, "Blood Furnace", 750, false)
	    	FIZ_AddInstance("Thrallmar", 7, 8, "Blood Furnace", 2700, true)
	    	FIZ_AddInstance("Thrallmar", 4, 8, "Shattered Halls", 1600, false)
	    	FIZ_AddQuest("Thrallmar", 4, 7, "[60] Hellfire Fortifications", 150)

	    	-- --- Shattrath ---
	    	-- Lower City
	    	FIZ_AddInstance("Lower City", 4, 5, "Auchenai Crypts", 750, false)
	    	FIZ_AddInstance("Lower City", 7, 8, "Auchenai Crypts", 750, true)
	    	FIZ_AddInstance("Lower City", 4, 5, "Sethekk Halls", 1250, false)
	    	FIZ_AddInstance("Lower City", 7, 8, "Sethekk Halls", 1250, true)
	    	FIZ_AddInstance("Lower City", 4, 8, "Shadow Labyrinth", 2000, false)
	    	FIZ_AddQuest("Lower City", 4, 5, "[65] The Outcast's Plight", 250, "Arakkoa Feather", 30)
	    	-- Sha'tari Skyguard
	    	FIZ_AddQuest("Sha'tari Skyguard", 4, 8, "[70] World of Shadows", 75, "Shadow Dust", 6)
	    	FIZ_AddQuest("Sha'tari Skyguard", 4, 8, "[70] Fires over Skettis", 350)
	    	FIZ_AddQuest("Sha'tari Skyguard", 4, 8, "[70] Escape from Skettis", 350)
	    	FIZ_AddQuest("Sha'tari Skyguard", 4, 8, "[70] Wrangle some Aether Rays!", 350)
	    	FIZ_AddQuest("Sha'tari Skyguard", 4, 8, "[70] Bombing Run", 500)
	    	FIZ_AddMob("Sha'tari Skyguard", 4, 6, "Skettis Kaliri", 2.5, "Terokkar Forest, Skettis")
	    	FIZ_AddMob("Sha'tari Skyguard", 4, 8, "Skettis, Talonpriests, Time-Lost Skettis", 5, "Terokkar Forest, Skettis")
	    	FIZ_AddMob("Sha'tari Skyguard", 4, 8, "Talonsworn Forest-Rager", 15, "Terokkar Forest, Skettis")
	    	-- The Aldor
	    	FIZ_AddQuest("The Aldor", 1, 3, "[64] Strained Supplies", 250, "Dreadfang Venom Sac", 8)
	    	FIZ_AddQuest("The Aldor", 4, 5, "[62] Marks of Kil'jaeden", 250, "Mark of Kil'jaeden", 10)
	    	FIZ_AddQuest("The Aldor", 5, 8, "[68] Marks of Sargeras", 250, "Mark of Sargeras", 10)
	    	FIZ_AddQuest("The Aldor", 5, 8, "[64] A Cleansing Light", 350, "Fel Armament", 1)
	    	-- The Scryers
	    	FIZ_AddQuest("The Scryers", 1, 3, "[64] Voren'thals Visions", 250, "Dampscale Basilisk Eye", 8)
	    	FIZ_AddQuest("The Scryers", 4, 5, "[62] Firewing Signets", 250, "Firewing Signet", 10)
	    	FIZ_AddQuest("The Scryers", 5, 8, "[68] Sunfury Signets", 250, "Sunfury Signet", 10)
	    	FIZ_AddQuest("The Scryers", 5, 8, "[64] Synthesis of Power", 350, "Arcane Tome", 1)
	    	-- The Sha'tar
	    	FIZ_AddQuest("The Sha'tar", 4, 5, "[62] Marks of Kil'jaeden", 125, "Mark of Kil'jaeden", 10)
	    	FIZ_AddQuest("The Sha'tar", 4, 5, "[68] Marks of Sargeras", 125, "Mark of Sargeras", 10)
	    	FIZ_AddQuest("The Sha'tar", 4, 5, "[62] Firewing Signets", 125, "Firewing Signet", 10)
	    	FIZ_AddQuest("The Sha'tar", 4, 5, "[68] Sunfury Signets", 125, "Sunfury Signet", 10)
	    	FIZ_AddQuest("The Sha'tar", 4, 5, "[64] Strained Supplies", 125, "Dreadfang Venom Sac", 8)
	    	FIZ_AddQuest("The Sha'tar", 4, 5, "[64] Voren'thals Visions", 125, "Dampscale Basilisk Eye", 8)
	    	FIZ_AddQuest("The Sha'tar", 4, 5, "[64] A Cleansing Light", 175, "Fel Armament", 1)
	    	FIZ_AddQuest("The Sha'tar", 4, 5, "[64] Synthesis of Power", 175, "Arcane Tome", 1)
		FIZ_AddInstance("The Sha'tar", 4, 8, "Mechanar", 1620, false)
		FIZ_AddInstance("The Sha'tar", 4, 8, "Mechanar", 3000, true)
		FIZ_AddInstance("The Sha'tar", 4, 8, "Botanica", 2000, false)
		FIZ_AddInstance("The Sha'tar", 4, 8, "Botanica", 3000, true)
		FIZ_AddInstance("The Sha'tar", 4, 8, "Arcatraz", 1800, false)
	end
end

function FIZ_AddMob(faction, from, to, name, rep, zone)
	if not faction then return end
	if not from then return end
	if not to then return end
	if not name then return end
	if not rep then return end
	if ((from<1) or (from>8)) then return end
	if ((to<1) or (to>8)) then return end
	if (from > to) then return end
	faction = string.lower(faction)

	for standing = from,to do
		if (not FIZ_FactionGain[faction]) then
			FIZ_FactionGain[faction] = {}
		end
		if (not FIZ_FactionGain[faction][standing]) then
			FIZ_FactionGain[faction][standing] = {}
		end
		if (not FIZ_FactionGain[faction][standing].mobs) then
			FIZ_FactionGain[faction][standing].mobs = {}
			FIZ_FactionGain[faction][standing].mobs.data = {}
			FIZ_FactionGain[faction][standing].mobs.count = 0
		else
			FIZ_FactionGain[faction][standing].mobs.count = FIZ_FactionGain[faction][standing].mobs.count + 1
		end
		FIZ_FactionGain[faction][standing].mobs.data[FIZ_FactionGain[faction][standing].mobs.count] = {}
		FIZ_FactionGain[faction][standing].mobs.data[FIZ_FactionGain[faction][standing].mobs.count].name = name
		FIZ_FactionGain[faction][standing].mobs.data[FIZ_FactionGain[faction][standing].mobs.count].rep = rep
		FIZ_FactionGain[faction][standing].mobs.data[FIZ_FactionGain[faction][standing].mobs.count].zone = zone

		FIZ_Debug("Added mob ["..name.."] for faction ["..faction.."] and standing ["..getglobal("FACTION_STANDING_LABEL"..standing).."]")
	end
end

function FIZ_AddQuest(faction, from, to, name, rep, item, num_item)
	if not faction then return end
	if not from then return end
	if not to then return end
	if not name then return end
	if not rep then return end
	if ((from<1) or (from>8)) then return end
	if ((to<1) or (to>8)) then return end
	if (from > to) then return end
	faction = string.lower(faction)

	for standing = from,to do
		if (not FIZ_FactionGain[faction]) then
			FIZ_FactionGain[faction] = {}
		end
		if (not FIZ_FactionGain[faction][standing]) then
			FIZ_FactionGain[faction][standing] = {}
		end
		if (not FIZ_FactionGain[faction][standing].quests) then
			FIZ_FactionGain[faction][standing].quests = {}
			FIZ_FactionGain[faction][standing].quests.data = {}
			FIZ_FactionGain[faction][standing].quests.count = 0
		else
			FIZ_FactionGain[faction][standing].quests.count = FIZ_FactionGain[faction][standing].quests.count + 1
		end
		FIZ_FactionGain[faction][standing].quests.data[FIZ_FactionGain[faction][standing].quests.count] = {}
		FIZ_FactionGain[faction][standing].quests.data[FIZ_FactionGain[faction][standing].quests.count].name = name
		FIZ_FactionGain[faction][standing].quests.data[FIZ_FactionGain[faction][standing].quests.count].rep = rep
		if (item and num_item) then
			FIZ_FactionGain[faction][standing].quests.data[FIZ_FactionGain[faction][standing].quests.count].item = item
			FIZ_FactionGain[faction][standing].quests.data[FIZ_FactionGain[faction][standing].quests.count].num_item = num_item
		end

		FIZ_Debug("Added quest ["..name.."] for faction ["..faction.."] and standing ["..getglobal("FACTION_STANDING_LABEL"..standing).."]")
	end
end

function FIZ_AddInstance(faction, from, to, name, rep, heroic)
	if not faction then return end
	if not from then return end
	if not to then return end
	if not name then return end
	if not rep then return end
	if ((from<1) or (from>8)) then return end
	if ((to<1) or (to>8)) then return end
	if (from > to) then return end
	faction = string.lower(faction)

	for standing = from,to do
		if (not FIZ_FactionGain[faction]) then
			FIZ_FactionGain[faction] = {}
		end
		if (not FIZ_FactionGain[faction][standing]) then
			FIZ_FactionGain[faction][standing] = {}
		end
		if (not FIZ_FactionGain[faction][standing].instance) then
			FIZ_FactionGain[faction][standing].instance = {}
			FIZ_FactionGain[faction][standing].instance.data = {}
			FIZ_FactionGain[faction][standing].instance.count = 0
		else
			FIZ_FactionGain[faction][standing].instance.count = FIZ_FactionGain[faction][standing].instance.count + 1
		end
		FIZ_FactionGain[faction][standing].instance.data[FIZ_FactionGain[faction][standing].instance.count] = {}
		FIZ_FactionGain[faction][standing].instance.data[FIZ_FactionGain[faction][standing].instance.count].name = name
		FIZ_FactionGain[faction][standing].instance.data[FIZ_FactionGain[faction][standing].instance.count].rep = rep
		if (heroic) then
			FIZ_FactionGain[faction][standing].instance.data[FIZ_FactionGain[faction][standing].instance.count].level = "Heroic"
		else
			FIZ_FactionGain[faction][standing].instance.data[FIZ_FactionGain[faction][standing].instance.count].level = "Normal"
		end

		FIZ_Debug("Added instance ["..name.."] for faction ["..faction.."] and standing ["..getglobal("FACTION_STANDING_LABEL"..standing).."]")
	end
end


-----------------------------------
-- _10_ Handle update            --
-----------------------------------
function FIZ_UpdateFaction()
end




--[[
    CollapseFactionHeader(index)   - Collapse a faction header row.
    ExpandFactionHeader(index)   - Expand a faction header row.
    FactionToggleAtWar(index)   - Toggle the At War flag for a faction.
    GetFactionInfo(index)   - Gets details for a specific faction/faction header.
    GetNumFactions()   - Returns the number of lines in the faction display.
    GetSelectedFaction()   - Returns the row index of the currently selected faction in reputation window. (New in 1.10)
    GetWatchedFactionInfo()   - Returns information about the currently watched faction. (New in 1.10)
    IsFactionInactive(index)   - Returns true if the faction is marked inactive. (New in 1.10)
    SetFactionActive(index)   - Remove a faction from inactive group. (New in 1.10)
    SetFactionInactive(index)   - Move a faction to inactive group. (New in 1.10)
    SetSelectedFaction(index)   - Sets the currently selected faction in reputation window. (New in 1.10)
    SetWatchedFactionIndex(index)   - Sets which faction should be watched in Blizzard reputation bar. (New in 1.10)
    UnitFactionGroup("unit")   - Returns the faction group id and name of the specified unit. (eg. "Alliance")   - string returned is localization-independent (used in filepath)

StandingId
0 - Unknown
1 - Hated	-42k .. -6k	36k
2 - Hostile	- 6k .. -3k	 3k
3 - Unfriendly	- 3k ..  0	 3k
4 - Neutral	  0  ..  3k	 3k
5 - Friendly	  3k ..  9k	 6k
6 - Honored	  9k .. 21k	12k
7 - Revered	 21k .. 42k	21k
8 - Exalted	 42k .. 43k	 1k

These values are defined globaly by the FACTION_STANDING_LABEL(x) variables, locale specific.
It's better to use them instead of relying on the hardcoded values above.

]]--